package sun.awt.shell;

import java.awt.Toolkit;
import java.util.Date;
import java.util.Iterator;
import java.util.Collections;
import java.util.concurrent.Callable;
import java.util.List;
import java.io.IOException;
import java.awt.Image;
import java.io.FileNotFoundException;
import java.util.Vector;
import java.io.ObjectStreamException;
import java.util.Comparator;
import java.io.File;

public abstract class ShellFolder extends File
{
    private static final String COLUMN_NAME = "FileChooser.fileNameHeaderText";
    private static final String COLUMN_SIZE = "FileChooser.fileSizeHeaderText";
    private static final String COLUMN_DATE = "FileChooser.fileDateHeaderText";
    protected ShellFolder parent;
    private static final ShellFolderManager shellFolderManager;
    private static final ShellFolder.Invoker invoker;
    private static final Comparator DEFAULT_COMPARATOR;
    private static final Comparator<File> FILE_COMPARATOR;
    
    ShellFolder(ShellFolder parent, String s) {
        super((s != null) ? s : "ShellFolder");
        this.parent = parent;
    }
    
    public boolean isFileSystem() {
        return !this.getPath().startsWith("ShellFolder");
    }
    
    protected abstract Object writeReplace() throws ObjectStreamException;
    
    @Override
    public String getParent() {
        if (this.parent == null && this.isFileSystem()) {
            return super.getParent();
        }
        if (this.parent != null) {
            return this.parent.getPath();
        }
        return null;
    }
    
    @Override
    public File getParentFile() {
        if (this.parent != null) {
            return this.parent;
        }
        if (this.isFileSystem()) {
            return super.getParentFile();
        }
        return null;
    }
    
    @Override
    public File[] listFiles() {
        return this.listFiles(true);
    }
    
    public File[] listFiles(boolean b) {
        File[] listFiles = super.listFiles();
        if (!b) {
            Vector<Object> vector = new Vector();
            for (int n = (listFiles == null) ? 0 : listFiles.length, i = 0; i < n; ++i) {
                if (!listFiles[i].isHidden()) {
                    vector.addElement(listFiles[i]);
                }
            }
            listFiles = (File[])(File[])vector.toArray(new File[vector.size()]);
            return listFiles;
        }
        return listFiles;
    }
    
    public abstract boolean isLink();
    
    public abstract ShellFolder getLinkLocation() throws FileNotFoundException;
    
    public abstract String getDisplayName();
    
    public abstract String getFolderType();
    
    public abstract String getExecutableType();
    
    @Override
    public int compareTo(File pathname) {
        if (pathname == null || !(pathname instanceof ShellFolder) || (pathname instanceof ShellFolder && ((ShellFolder)pathname).isFileSystem())) {
            if (this.isFileSystem()) {
                return super.compareTo(pathname);
            }
            return -1;
        }
        else {
            if (this.isFileSystem()) {
                return 1;
            }
            return this.getName().compareTo(pathname.getName());
        }
    }
    
    public Image getIcon(boolean b) {
        return null;
    }
    
    public static ShellFolder getShellFolder(File file) throws FileNotFoundException {
        if ((file instanceof ShellFolder)) {
            return (ShellFolder)file;
        }
        if (!file.exists()) {
            throw new FileNotFoundException();
        }
        return ShellFolder.shellFolderManager.createShellFolder(file);
    }
    
    public static Object get(String s) {
        return ShellFolder.shellFolderManager.get(s);
    }
    
    public static boolean isComputerNode(File file) {
        return ShellFolder.shellFolderManager.isComputerNode(file);
    }
    
    public static boolean isFileSystemRoot(File file) {
        return ShellFolder.shellFolderManager.isFileSystemRoot(file);
    }
    
    public static File getNormalizedFile(File file) throws IOException {
        File canonicalFile = file.getCanonicalFile();
        if (file.equals(canonicalFile)) {
            return canonicalFile;
        }
        return new File(file.toURI().normalize());
    }
    
    public static void sort(final List<? extends File> list) {
        if (list == null || list.size() <= 1) {
            return;
        }
        invoke(new Callable<Void>() {
            @Override
            public Void call() {
                File file = null;
                for (File file2 : list) {
                    File parentFile = file2.getParentFile();
                    if (parentFile == null || !(file2 instanceof ShellFolder)) {
                        file = null;
                        break;
                    }
                    if (file == null) {
                        file = parentFile;
                    }
                    else {
                        if (file != parentFile && !file.equals(parentFile)) {
                            file = null;
                            break;
                        }
                        continue;
                    }
                }
                if ((file instanceof ShellFolder)) {
                    ((ShellFolder)file).sortChildren(list);
                }
                else {
                    Collections.sort(list, ShellFolder.FILE_COMPARATOR);
                }
                return null;
            }
        });
        return;
    }
    
    public void sortChildren(final List<? extends File> list) {
        invoke(new Callable<Void>() {
            @Override
            public Void call() {
                Collections.sort(list, ShellFolder.FILE_COMPARATOR);
                return null;
            }
        });
    }
    
    @Override
    public boolean isAbsolute() {
        return !this.isFileSystem() || super.isAbsolute();
    }
    
    @Override
    public File getAbsoluteFile() {
        return this.isFileSystem() ? super.getAbsoluteFile() : this;
    }
    
    @Override
    public boolean canRead() {
        return !this.isFileSystem() || super.canRead();
    }
    
    @Override
    public boolean canWrite() {
        return this.isFileSystem() && super.canWrite();
    }
    
    @Override
    public boolean exists() {
        return !this.isFileSystem() || isFileSystemRoot(this) || super.exists();
    }
    
    @Override
    public boolean isDirectory() {
        return !this.isFileSystem() || super.isDirectory();
    }
    
    @Override
    public boolean isFile() {
        return this.isFileSystem() ? super.isFile() : (!this.isDirectory());
    }
    
    @Override
    public long lastModified() {
        return this.isFileSystem() ? super.lastModified() : 0L;
    }
    
    @Override
    public long length() {
        return this.isFileSystem() ? super.length() : 0L;
    }
    
    @Override
    public boolean createNewFile() throws IOException {
        return this.isFileSystem() && super.createNewFile();
    }
    
    @Override
    public boolean delete() {
        return this.isFileSystem() && super.delete();
    }
    
    @Override
    public void deleteOnExit() {
        if (this.isFileSystem()) {
            super.deleteOnExit();
        }
    }
    
    @Override
    public boolean mkdir() {
        return this.isFileSystem() && super.mkdir();
    }
    
    @Override
    public boolean mkdirs() {
        return this.isFileSystem() && super.mkdirs();
    }
    
    @Override
    public boolean renameTo(File dest) {
        return this.isFileSystem() && super.renameTo(dest);
    }
    
    @Override
    public boolean setLastModified(long lastModified) {
        return this.isFileSystem() && super.setLastModified(lastModified);
    }
    
    @Override
    public boolean setReadOnly() {
        return this.isFileSystem() && super.setReadOnly();
    }
    
    @Override
    public String toString() {
        return this.isFileSystem() ? super.toString() : this.getDisplayName();
    }
    
    public static ShellFolderColumnInfo[] getFolderColumns(File file) {
        ShellFolderColumnInfo[] folderColumns = null;
        if ((file instanceof ShellFolder)) {
            folderColumns = ((ShellFolder)file).getFolderColumns();
        }
        if (folderColumns == null) {
            folderColumns = new ShellFolderColumnInfo[] { new ShellFolderColumnInfo("FileChooser.fileNameHeaderText", Integer.valueOf(150), Integer.valueOf(10), true, null, ShellFolder.FILE_COMPARATOR), new ShellFolderColumnInfo("FileChooser.fileSizeHeaderText", Integer.valueOf(75), Integer.valueOf(4), true, null, ShellFolder.DEFAULT_COMPARATOR, true), new ShellFolderColumnInfo("FileChooser.fileDateHeaderText", Integer.valueOf(130), Integer.valueOf(10), true, null, ShellFolder.DEFAULT_COMPARATOR, true) };
            return folderColumns;
        }
        return folderColumns;
    }
    
    public ShellFolderColumnInfo[] getFolderColumns() {
        return null;
    }
    
    public static Object getFolderColumnValue(File file, int n) {
        if ((file instanceof ShellFolder)) {
            Object folderColumnValue = ((ShellFolder)file).getFolderColumnValue(n);
            if (folderColumnValue != null) {
                return folderColumnValue;
            }
        }
        if (file == null || !file.exists()) {
            return null;
        }
        switch (n) {
            case 0: {
                return file;
            }
            case 1: {
                return file.isDirectory() ? null : Long.valueOf(file.length());
            }
            case 2: {
                if (isFileSystemRoot(file)) {
                    return null;
                }
                long lastModified = file.lastModified();
                return (lastModified == 0L) ? null : new Date(lastModified);
            }
            default: {
                return null;
            }
        }
    }
    
    public Object getFolderColumnValue(int n) {
        return null;
    }
    
    public static <T> T invoke(Callable<T> callable) {
        try {
            return invoke(callable, RuntimeException.class);
        }
        catch (InterruptedException ex) {
            return null;
        }
    }
    
    public static <T, E extends Throwable> T invoke(Callable<T> callable, Class<E> clazz) throws InterruptedException, E, Throwable {
        try {
            return ShellFolder.invoker.invoke(callable);
        }
        catch (Exception ex) {
            if ((ex instanceof RuntimeException)) {
                throw (RuntimeException)ex;
            }
            if ((ex instanceof InterruptedException)) {
                Thread.currentThread().interrupt();
                throw (InterruptedException)ex;
            }
            if (clazz.isInstance(ex)) {
                throw (Throwable)clazz.cast(ex);
            }
            throw new RuntimeException("Unexpected error", ex);
        }
    }
    
    static {
        String name = (String)Toolkit.getDefaultToolkit().getDesktopProperty("Shell.shellFolderManager");
        Class<Object> forName = null;
        try {
            forName = Class.forName(name, false, null);
            if (!ShellFolderManager.class.isAssignableFrom(forName)) {
                forName = null;
            }
        }
        catch (ClassNotFoundException ex) {}
        catch (NullPointerException ex2) {}
        catch (SecurityException ex3) {}
        if (forName == null) {
            forName = ShellFolderManager.class;
        }
        try {
            shellFolderManager = (ShellFolderManager)forName.newInstance();
        }
        catch (InstantiationException ex4) {
            throw new Error("Could not instantiate Shell Folder Manager: " + forName.getName());
        }
        catch (IllegalAccessException ex5) {
            throw new Error("Could not access Shell Folder Manager: " + forName.getName());
        }
        invoker = ShellFolder.shellFolderManager.createInvoker();
        DEFAULT_COMPARATOR = new Comparator() {
            @Override
            public int compare(Object o, Object o2) {
                int compareTo;
                if (o == null && o2 == null) {
                    compareTo = 0;
                }
                else if (o != null && o2 == null) {
                    compareTo = 1;
                }
                else if (o == null && o2 != null) {
                    compareTo = -1;
                }
                else if ((o instanceof Comparable)) {
                    compareTo = ((Comparable)o).compareTo(o2);
                }
                else {
                    compareTo = 0;
                }
                return compareTo;
            }
        };
        FILE_COMPARATOR = new Comparator<File>() {
            @Override
            public int compare(File file, File file2) {
                ShellFolder shellFolder = null;
                ShellFolder shellFolder2 = null;
                if ((file instanceof ShellFolder)) {
                    shellFolder = (ShellFolder)file;
                    if (shellFolder.isFileSystem()) {
                        shellFolder = null;
                    }
                }
                if ((file2 instanceof ShellFolder)) {
                    shellFolder2 = (ShellFolder)file2;
                    if (shellFolder2.isFileSystem()) {
                        shellFolder2 = null;
                    }
                }
                if (shellFolder != null && shellFolder2 != null) {
                    return shellFolder.compareTo(shellFolder2);
                }
                if (shellFolder != null) {
                    return -1;
                }
                if (shellFolder2 != null) {
                    return 1;
                }
                String name = file.getName();
                String name2 = file2.getName();
                int compareToIgnoreCase = name.compareToIgnoreCase(name2);
                if (compareToIgnoreCase != 0) {
                    return compareToIgnoreCase;
                }
                return name.compareTo(name2);
            }
        };
    }
    
    public interface Invoker
    {
        public abstract <T> T invoke(Callable<T> p0) throws Exception;
    }
}
